"
 COPYRIGHT (c) 1997 by eXept Software AG / Thomas Zwick
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice. This software may not
 be provided or otherwise made available to, or used by, any
 other person. No title to or ownership of the software is
 hereby transferred.
"

ImageView subclass:#ImageEditView
	instanceVariableNames:'magnification selectColors imageReaderClass resourceClass
		resourceSelector editMode mouseKeyColorMode undoImage modified
		masterApplication'
	classVariableNames:'Clipboard GridMagnification'
	poolDictionaries:''
	category:'Views-Misc'
!

!ImageEditView class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by eXept Software AG / Thomas Zwick
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice. This software may not
 be provided or otherwise made available to, or used by, any
 other person. No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    The ImageEditView is a view class which can be used by applications
    like the Image Editor for modifying or inspecting images.

    [see also:]
        ImageEditor Image

    [author:]
        Thomas Zwick
"
! !

!ImageEditView methodsFor:'accessing'!

magnification

    ^magnification

!

magnification: aPoint

    magnification ~= aPoint
    ifTrue:
    [
        magnification := aPoint asPoint.
        self scrollToTopLeft.
        self contentsChanged.
        self invalidate
    ]
!

masterApplication

    ^masterApplication
!

masterApplication: anApplicationModel

    masterApplication := anApplicationModel
!

resourceClass

    ^resourceClass
!

resourceClass: aClassOrSymbol

    resourceClass := aClassOrSymbol isClass ifTrue: [aClassOrSymbol name] ifFalse: [aClassOrSymbol asSymbol]
!

resourceMessage

    ^resourceClass, ' ', resourceSelector
!

resourceMessage: aString

    (aString isString and: [aString trimBlanks size > 0])
    ifTrue:
    [
        resourceClass := (aString readStream upTo: Character space) asSymbol.
        resourceSelector := (aString copy reverse readStream upTo: Character space) reverse asSymbol
    ]
    ifFalse:
    [
        ^nil
    ]

   
!

resourceSelector

    ^resourceSelector
!

resourceSelector: aStringOrSymbol

    resourceSelector := aStringOrSymbol asSymbol
!

selectColor: anArrayTwoColors

    selectColors := anArrayTwoColors
!

selectedColor

   ^selectColors at: mouseKeyColorMode
!

selectedColor: aColor

    selectColors at: mouseKeyColorMode put: aColor
! !

!ImageEditView methodsFor:'drawing'!

fillFramedRectangle: aRectangle

    self fillRectangle: aRectangle.
    self drawFramesIn: aRectangle
!

redrawImageX:x y:y width:w height:h

    |ih iw magX magY minX maxX minY maxY color lastColor lastY runW x0 xI yI maskColor sizeOfMaskPoint|

    ih := image height.
    iw := image width.
    magX := magnification x.
    magY := magnification y.

    minX := (x // magX - 1) max: 0.
    minX >= iw ifTrue:[minX := iw - 1].
    minY := (y // magY - 1) max: 0.
    minY >= ih ifTrue:[minY := ih - 1].
    maxX := (x + w) // magX + 1.
    maxX > iw ifTrue:[maxX := iw].
    maxY := (y + h) // magY + 1.
    maxY > ih ifTrue:[maxY := ih].

    lastY := -1.

    x0 := minX.
    runW := 0.
    maskColor := false.
    sizeOfMaskPoint := magnification//3.
    image colorsFromX: minX y: minY toX: maxX-1 y: maxY-1 do:
    [:xx :yy :color|
        yy ~~ lastY
        ifTrue:
        [
            runW ~~ 0
            ifTrue:
            [
                |origin|
                origin := (x0 * magX + margin)@(lastY * magY + margin).
                self fillFramedRectangle: (origin extent: (runW@magY)).                    
                maskColor ifTrue:
                [
                    self xoring: [self fillRectangle: (origin + sizeOfMaskPoint extent: sizeOfMaskPoint)]
                ].
                runW := 0.
            ]. 
            x0 := xx.
            lastY := yy.
        ]. 

        color ~~ lastColor
        ifTrue:
        [
            runW ~~ 0
            ifTrue:
            [
                |origin|
                origin := (x0 * magX + margin)@(yy * magY + margin).
                self fillFramedRectangle: (origin extent: (runW@magY)).
                maskColor ifTrue:
                [
                    self xoring: [self fillRectangle: (origin + sizeOfMaskPoint extent: sizeOfMaskPoint)]
                ].
                runW := 0.
            ].

            self paint: (lastColor := color).
            image mask notNil ifTrue:
            [  
                maskColor := false.
                (image mask colorAt: xx@yy) = Color black ifTrue:
                [
                    self paint: (lastColor := self viewBackground).
                    maskColor := true.
                ].
                lastColor := nil.
            ].
            runW := 0.
            x0 := xx.
        ].  
        runW := runW + magX
    ].
    runW ~~ 0 ifTrue:
    [
        |origin|
        origin := (x0 * magX + margin)@(lastY * magY + margin).
        self fillFramedRectangle: (origin extent: runW@magY).
        maskColor ifTrue:
        [
            self xoring: [self fillRectangle: (origin + sizeOfMaskPoint extent: sizeOfMaskPoint)]
        ].
        runW := 0.
    ].
!

redrawX:x y:y width:w height:h

    |ih iw xI yI|

    magnification = (1@1) ifTrue: [^super redrawX:x y:y width:w height:h].

    image isNil ifTrue:[^self].

    self clippingRectangle: (x@y extent: w@h).

    self redrawImageX:x y:y width:w height:h.

    "/ right of image ?
    adjust == #center ifTrue:
    [
        xI := (width - ih) // 2 - margin.
        yI := (height - iw) // 2 - margin.
    ]
    ifFalse:
    [
        xI := yI := margin
    ].
    (x + w - 1) > (xI + (magnification x * image width)) ifTrue:
    [
        self clearRectangleX:(xI + (magnification x * image width))
                           y:y
                       width:(x + w - (magnification x * image width) - xI)
                      height:h
    ].
    (y + h - 1) > (yI + (magnification y * image height)) ifTrue:
    [
        self clearRectangleX:margin
                           y:(yI + (magnification y * image height))
                       width:w
                      height:(y + h - (magnification y * image height) - yI)  
    ].
    self drawFrame.
    self clippingRectangle: nil.
! !

!ImageEditView methodsFor:'edit modes'!

editMode

    ^editMode ? (editMode := 'point')
!

editMode:aMode

    editMode := aMode
!

mouseKeyColorMode

    ^mouseKeyColorMode printString
!

mouseKeyColorMode:aMode

    mouseKeyColorMode := aMode asInteger
! !

!ImageEditView methodsFor:'event handling'!

buttonMotion:state x:x y:y

    self drawCursorAt: x@y.
    state ~~ 0
    ifTrue:
    [
        self selectedColor notNil & image notNil & (self imageContainsPoint: x@y) & (editMode = 'point')
            ifTrue: [^self pointAt: x@y]
    ]
!

buttonPress:button x:x y:y

    self drawCursorAt: x@y.
    self selectedColor notNil & image notNil & (self imageContainsPoint: x@y)
    ifTrue:
    [    
        undoImage := image copy.
        mouseKeyColorMode := button.
        self perform: (editMode, 'At:') asSymbol with: x@y
    ]
!

buttonRelease:state x:x y:y

    Cursor normal show
!

pointerLeave:state

    super pointerLeave:state.

    self drawLabel: ''.
    Cursor normal show
! !

!ImageEditView methodsFor:'image editing'!

boxAt: aPoint

    |choosedBox imageBox|
    choosedBox := self drawRectangleStartingAt: aPoint emphasis: #box.
    image rectangle: (imageBox := choosedBox origin//magnification extent: (choosedBox extent//magnification)) withColor: self selectedColor.
    image restored.
    self redraw: (choosedBox expandedBy: 1).
    masterApplication notNil
    ifTrue:
    [
        masterApplication imagePreView redraw: (imageBox expandedBy: 1)
    ].
    modified := true
!

copyAt: aPoint

    |choosedBox|
    choosedBox := self drawRectangleStartingAt: aPoint emphasis: #inverseFilledBox.
    Clipboard := image subImageIn: (choosedBox origin//magnification extent: (choosedBox extent//magnification)).
    self redraw: (choosedBox expandedBy: 1)    

!

fillAt: aPoint

    Cursor execute showWhile:
    [
        |filledPoints|
        filledPoints := image floodFillAt: aPoint//magnification withColor: self selectedColor.
        image restored.
        filledPoints size < 300
        ifTrue:
        [
            filledPoints do: [:p| self redraw: ((p * magnification extent: magnification) expandedBy: 1)]
        ]
        ifFalse:
        [
            self invalidate
        ].
        masterApplication notNil
        ifTrue:
        [
            masterApplication imagePreView invalidate
        ].
        modified := true
    ]
!

filledBoxAt: aPoint

    |choosedBox imageBox|
    choosedBox := self drawRectangleStartingAt: aPoint emphasis: #filledBox.
    image fillRectangle: (imageBox := choosedBox origin//magnification extent: (choosedBox extent//magnification)) withColor: self selectedColor.
    image restored.
    self redraw: (choosedBox expandedBy: 1).
    masterApplication notNil
    ifTrue:
    [
        masterApplication imagePreView redraw: (imageBox expandedBy: 1)

    ].
    modified := true
!

flipHorizontal

    undoImage := image copy.
    self image: image flipHorizontal

!

flipVertical

    undoImage := image copy.
    self image: image flipVertical

!

magnifyImage

    |box newSize|
    box := EnterBox new.
    box title:'Image new size:'.
    box okText:'OK'.
    box abortText:'Cancel'.
    box initialText:image extent printString.
    box showAtPointer.
    (box accepted and: [(newSize := Object readFromString: box contents onError:nil) notNil])
    ifTrue:
    [
        undoImage := image copy.
        self image: (image magnifiedBy: newSize/image extent)
    ]
!

negativeImage

    undoImage := image copy.
    self image: image negative

!

pasteAt: aPoint

    Object errorSignal handle:
    [:ex|
        self warn: 'Pasting at selected point failed!!'
    ] 
    do:
    [   
        |imagePoint imageBox|
        imagePoint := aPoint//magnification.
        image copyFrom: Clipboard x:0 y:0 toX: imagePoint x y: imagePoint y width: Clipboard width height: Clipboard height.
        image restored.
        self redraw: (imageBox := (imagePoint * magnification extent: (Clipboard extent * magnification)) expandedBy: 1@1).
        masterApplication notNil
        ifTrue:
        [
            masterApplication imagePreView redraw: (imageBox expandedBy: 1)
        ].
        modified := true
    ]
!

pointAt: aPoint

    |imagePoint imageBox|
    imagePoint := aPoint//magnification.
    image atImageAndMask: imagePoint put: self selectedColor.
    image restored.
    self redraw: (imageBox := (imagePoint * magnification extent: magnification) expandedBy: 1).
    masterApplication notNil
    ifTrue:
    [             
        masterApplication imagePreView redraw: ((imagePoint + 1) extent: 1@1)
    ].
    modified := true
!

resizeImage

    |box newSize|
    box := EnterBox new.
    box title:'Image new size:'.
    box okText:'OK'.
    box abortText:'Cancel'.
    box initialText:image extent printString.
    box showAtPointer.
    (box accepted and: [(newSize := Object readFromString: box contents onError:nil) notNil])
    ifTrue:
    [
        |newImage|
        undoImage := image copy.
        newImage := image class width: newSize x height: newSize y depth: image depth.
        newImage photometric:image photometric.
        newImage colorMap:image colorMap copy.
        newImage bits: (ByteArray new: newImage bytesPerRow * newSize y).

        image mask notNil
        ifTrue: 
        [
            |newMaskImage|
            newMaskImage := Depth1Image width: newSize x height: newSize y.
            newMaskImage photometric: image mask photometric.
            newMaskImage colorMap: image mask colorMap copy.
            newMaskImage bits:(ByteArray new: newMaskImage bytesPerRow * newSize y).
            newImage mask: newMaskImage
        ].

        newImage copyFrom:image x:0 y:0 toX:0 y:0 width: (image width min:newSize x) height: (image height min:newSize y).
        self image: newImage
    ]
!

rotateImage

    |box rotation|
    box := EnterBox new.
    box title:'Image new rotation:'.
    box okText:'OK'.
    box abortText:'Cancel'.
    box initialText: '0'.
    box showAtPointer.
    (box accepted and: [(rotation := Object readFromString: box contents onError:nil) notNil])
    ifTrue:
    [   Object errorSignal handle:
        [:ex|
            self warn: 'Image rotation failed.\' withCRs, 'An increase of image depth could help.'.
        ] 
        do:
        [   
            undoImage := image copy.
            self image: (image hardRotated: rotation)
        ]
    ]
!

undo

    undoImage notNil
    ifTrue:
    [
        modified := false.
        self image: undoImage.
        self invalidate
    ]
! !

!ImageEditView methodsFor:'image emphasis'!

drawCursorAt: aPoint

    self drawLabel: ''.
    image isNil ifTrue: [^Cursor stop show].
    ((0@0 extent: image extent * magnification) containsPoint: aPoint)
         ifFalse:[Cursor stop show]
         ifTrue: [self drawLabel: aPoint//magnification.Cursor crossHair show].

!

drawFrame

    self paint:Color black.
    self lineWidth: (magnification x//3 min: 3).
    self displayRectangle: ((0@0) extent:(image extent * magnification) + 2).
    self lineWidth:1.
!

drawFramesIn: aRectangle

    magnification > GridMagnification
    ifTrue:
    [   
        |origin lineStartingPoint lineEndingPoint oldColor|
        origin := aRectangle origin - 1.
        lineStartingPoint := origin + (0@magnification y).
        lineEndingPoint   := lineStartingPoint + (aRectangle width@0).
        oldColor := self paint.
        self xoring:
        [
            self displayLineFrom: lineStartingPoint to: lineEndingPoint.
            lineStartingPoint x to: lineStartingPoint x + aRectangle width - magnification x by: magnification x do:
            [:x|   
                self displayLineFrom: x@(origin y) to: x@(origin y + magnification y)
            ].
        ].
        self paint: oldColor.
    ]



!

drawLabel: aLabel

    (masterApplication respondsTo: #coordLabel)
    ifTrue:
    [                    
       masterApplication coordLabel label: aLabel printString
    ]
!

drawRectangleStartingAt: aPoint emphasis: emphasis

    |currentPoint currentExtent firstPoint lastCurrentPoint gridCorrection p|
    firstPoint := currentPoint := lastCurrentPoint := aPoint//magnification*magnification.
    magnification > GridMagnification ifFalse: [gridCorrection := 0] ifTrue: [gridCorrection := 1].
    [device anyButtonPressed]
    whileTrue:
    [   
        currentPoint := (0@0) max: (image extent * magnification min: (p := self translation negated + (device translatePoint: self sensor mousePoint from:device rootView id to:self id))).
        currentPoint := currentPoint//magnification*magnification.
        currentExtent := (firstPoint - currentPoint) abs.
        self drawCursorAt: p.

        currentPoint ~= lastCurrentPoint ifTrue:
        [          
            emphasis = #inverseFilledBox
            ifTrue:
            [
                self redraw: ((firstPoint min: lastCurrentPoint) - 1 extent: (firstPoint - lastCurrentPoint) abs + 2).
                self xoring: [self fillRectangle: ((firstPoint min: currentPoint) + 1 extent: currentExtent - gridCorrection)]
            ].
            emphasis = #box
            ifTrue:
            [
                |origin extent lineWidthY lineWidthX|
                origin := (firstPoint min: lastCurrentPoint) - 1.
                extent := (firstPoint - lastCurrentPoint) abs + 2.
                lineWidthY := extent y min: (magnification y + 2).
                lineWidthX := extent x min: (magnification x + 2).
                self redraw: (origin extent: (extent x@lineWidthY)).
                self redraw: ((origin x@(origin y + extent y - lineWidthY)) extent: (extent x@lineWidthY)).
                self redraw: ((origin x@(origin y + lineWidthY)) extent: (lineWidthX@(0 max: (extent y - (lineWidthY * 2))))).
                self redraw: (((origin x + extent x - lineWidthX)@(origin y + lineWidthY)) extent: (lineWidthX@(extent y - (lineWidthY * 2)))).
                self selectedColor ~= Color noColor
                    ifTrue: [self paint: self selectedColor]
                    ifFalse: [self paint: self viewBackground].
                origin := (firstPoint min: currentPoint) + 1.
                extent := currentExtent - gridCorrection.
                lineWidthY := extent y min: magnification y.
                lineWidthX := extent x min: magnification x.
                (lineWidthY > 0) & (lineWidthX > 0)
                ifTrue:
                [
                    self fillRectangle: (origin extent: (extent x@lineWidthY)).
                    self fillRectangle: ((origin x@(origin y + extent y - lineWidthY)) extent: (extent x@lineWidthY)).
                    self fillRectangle: ((origin x@(origin y + lineWidthY)) extent: (lineWidthX@(0 max: (extent y - (lineWidthY * 2))))).
                    self fillRectangle: (((origin x + extent x - lineWidthX)@(origin y + lineWidthY)) extent: (lineWidthX@(extent y - (lineWidthY * 2)))).
                ]
            ].
            emphasis = #filledBox
            ifTrue:
            [
                self redraw: ((firstPoint min: lastCurrentPoint) - 1 extent: (firstPoint - lastCurrentPoint) abs + 2).
                self selectedColor ~= Color noColor
                    ifTrue: [self paint: self selectedColor]
                    ifFalse: [self paint: self viewBackground].
                self fillRectangle: ((firstPoint min: currentPoint) + 1 extent: currentExtent - gridCorrection).
            ].
        ]. 
        lastCurrentPoint := currentPoint.
    ].

    ^((1@1) max: (firstPoint min: currentPoint)) extent: (firstPoint - currentPoint) abs
! !

!ImageEditView methodsFor:'image setting'!

image:anImage

    (anImage isImage and: [image isNil or: [self checkModified]])
    ifTrue:
    [    
        masterApplication notNil ifTrue: [masterApplication imagePreView image: anImage].
        super image: anImage.
        ^self
    ]
    ifFalse:
    [
        super image: nil.
        ^nil
    ]
!

loadFromFile: aFileName

    aFileName isNil ifTrue: [^nil].
    Object errorSignal handle:
    [:exeption|
        modified := false.
        self warn: exeption errorString.
        ^nil
    ] 
    do:
    [ 
        |imageFromFile|
        (imageFromFile := Image fromFile: aFileName) notNil
        ifTrue:
        [
            self image: imageFromFile. 
            imageReaderClass := ImageReader allSubclasses
                detect: [:cls| cls isValidImageFile: aFileName]
                ifNone: [self error: 'Unknown image file format!!']
        ]
        ifFalse:
        [
            self error: 'Image on file not found!!'
        ]
    ]
!

loadFromMessage: aMessage

    (self resourceMessage: aMessage) isNil
    ifTrue:
    [
        ^nil
    ].

    ^self loadfromClass: resourceClass andSelector: resourceSelector

   
!

loadfromClass: aClassOrSymbol andSelector: aStringOrSymbol

    |aClass| 

    imageReaderClass := nil.
    self resourceClass: aClassOrSymbol.
    self resourceSelector: aStringOrSymbol.

    ((aClass := Smalltalk at: resourceClass) isClass and: 
    [aClass class implements: resourceSelector])
    ifTrue:
    [ 
        ^self image: (aClass perform: resourceSelector) copy
    ]
    ifFalse:
    [
        modified := false.
        ^nil
    ]

! !

!ImageEditView methodsFor:'initialization'!

initialize

    super initialize.

    self enableMotionEvents.
    magnification := 1@1.
    GridMagnification := GridMagnification ? (8@8).
    modified := false.
    mouseKeyColorMode := 1.
    resourceClass := resourceSelector := ''.
    selectColors := Array with: nil with: nil.
! !

!ImageEditView methodsFor:'printing & storing'!

loadFromInClassesOf: aSuperclassOrSymbol

    ^self loadFromMessage: 
        (ResourceSelectionBrowser
            request: 'Load Image From Class'
            onSuperclass: aSuperclassOrSymbol
            andClass: self resourceClass
            andSelector: self resourceSelector
            withResourceTypes: #(image fileImage))
            

!

print

    image notNil 
    ifTrue: 
    [
        |stream|
        Printer supportsPostscript ifFalse:
        [
            ^self warn:'No postscript printer detected!!'
        ].
        stream := Printer newNative.
        stream isNil ifTrue:
        [
            ^self warn:'Cannot open printer stream!!'
        ].
        self withWaitCursorDo:
        [
            |psgc|
            psgc := PSGraphicsContext on:stream.
            psgc displayForm: (image magnifiedBy: magnification) x:0 y:0.
            psgc close
        ]
    ]
!

save

    self saveImageOrMask: #image.

!

saveAs

    self saveImageFileAs
!

saveImageFileAs

    self saveImageFileAs:
        (FileSelectionBrowser
            request: 'Save Image As'
            pathName: self image fileName
            fileFilters: FileSelectionBrowser saveImageFileNameFilters)
!

saveImageFileAs: aFileName

    aFileName isNil ifTrue: [^nil].
    image notNil
    ifTrue:
    [
        image fileName: aFileName.
        self saveImageOrMask: #image
    ]
    ifFalse:
    [
        self warn: 'No image or file name for saving detected!!'
    ]
!

saveImageMaskFileAs

    self saveImageMaskFileAs:
        (FileSelectionBrowser
            request: 'Save Image Mask As'
            pathName: self image fileName 
            fileFilters: FileSelectionBrowser saveImageFileNameFilters)
!

saveImageMaskFileAs: aFileName

    aFileName isNil ifTrue: [^nil].
    (image notNil and: [image mask notNil])
    ifTrue:
    [
        image mask fileName: aFileName.
        self saveImageOrMask: #mask
    ]
    ifFalse:
    [
        self warn: 'No image or image mask detected!!'
    ]
!

saveImageOrMask: what

    Object errorSignal handle:
    [:ex|
        self warn: ex errorString.
        ^nil                                 
    ] 
    do:
    [   
        |fileName|
        image isNil ifTrue: [^self error: 'No image to save!!'].
        what = #image
        ifTrue:
        [   
            image fileName isNil ifTrue: [^self error: 'No file name for image detected!!'].
            fileName := image fileName asFilename
        ].
        what = #mask
        ifTrue:
        [   
            image mask isNil ifTrue: [^self error: 'No image mask to save!!'].
            image mask fileName isNil ifTrue: [^self error: 'No file name for image mask detected!!'].
            fileName := image mask fileName asFilename
        ].
        fileName name size = 0 ifTrue: [^self error: 'No file name detected!!'].
        (fileName suffix = 'tiff') | (fileName suffix = 'tif') ifTrue: [imageReaderClass := TIFFReader].
        fileName suffix = 'xpm' ifTrue: [imageReaderClass := XPMReader].
        fileName suffix = 'xbm' ifTrue: [imageReaderClass := XBMReader].
        fileName suffix = 'gif' ifTrue: [imageReaderClass := GIFReader].
        (fileName suffix = 'jpg') | (fileName suffix = 'jpeg') ifTrue: [imageReaderClass := JPEGReader].
        imageReaderClass isNil ifTrue: [imageReaderClass := XPMReader. image fileName: image fileName, '.xpm'].
        what = #image ifTrue: [image saveOn: image fileName using: imageReaderClass. modified := false].
        what = #mask ifTrue: [image mask saveOn: image mask fileName using: imageReaderClass].
    ]   

       
!

saveMethod

    Object errorSignal handle:
    [:ex|
        self warn: ex errorString.
        ^nil                                 
    ] 
    do:
    [   
        |imageStoreStream cls|  
        (self resourceSelector trimBlanks size = 0) | (cls := Smalltalk at: self resourceClass) isClass not 
            ifTrue: [^self error: 'No class and selector for image detected!!'].
        self image storeOn: (imageStoreStream := WriteStream on: '').
        ByteCodeCompiler 
            compile: (self resourceSelector,
                '\' withCRs, '    "ImageEditor openOnClass:self andSelector:#', self resourceSelector, '"',
                '\\' withCRs, '    <resource: #image>\' withCRs, '    ^', imageStoreStream contents)
            forClass: cls class inCategory: 'resources'.
        modified := false.
    ]
!

saveMethodAs

    (self resourceMessage:
        (ResourceSelectionBrowser
            request: 'Save On Image Resource Selector'
            onSuperclass: #ApplicationModel
            andClass: self resourceClass
            andSelector: self resourceSelector
            withResourceTypes: #(image fileImage))) notNil
    ifTrue:
    [   
        ^self saveMethod
    ].  
    ^nil
! !

!ImageEditView methodsFor:'queries'!

heightOfContents

    image isNil ifTrue:[^0].
    ^(image height * magnification y) rounded
!

imageContainsPoint: aPoint

    ^image notNil and:
        [((0@0 corner:(image extent) - 1) containsPoint: (((aPoint - margin + 1) / magnification) floor))]
!

widthOfContents

    image isNil ifTrue:[^0].
    ^(image width * magnification x) rounded
! !

!ImageEditView methodsFor:'release'!

destroy

    Clipboard := undoImage := nil.

    super destroy

! !

!ImageEditView methodsFor:'testing'!

checkModified

    modified
    ifTrue:
    [
        ((YesNoBox title:'Image was modified!!')        
            noText:'Cancel';
            yesText:'Waste it and proceed';
            showAtPointer;
            accepted) ifFalse: [^false].
        modified := false
    ].
    ^true
! !

!ImageEditView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/ImageEditView.st,v 1.58 1998-01-27 12:17:37 tz Exp $'
! !
